@@ -144,6 +144,18 @@ impl DepthFirstTraverser {
             }
         }
     }
+
+    /// Returns the next event without advancing the iterator.
+    #[must_use]
+    pub(crate) fn peek(&self) -> Option<DftEvent> {
+        self.next.map(|(forward, _back)| forward)
+    }
+
+    /// Returns the backward next event without advancing the iterator.
+    #[must_use]
+    pub(crate) fn peek_back(&self) -> Option<DftEvent> {
+        self.next.map(|(_fwd, back)| back)
+    }
 }
 
 /// Ancestors traverser.
@@ -183,6 +195,13 @@ impl AncestorsTraverser {
 
         Some(next)
     }
+
+    /// Returns the next event without advancing the iterator.
+    #[inline]
+    #[must_use]
+    pub(crate) fn peek(&self) -> Option<NodeId> {
+        self.next
+    }
 }
 
 /// Double-ended siblings traverser.
@@ -312,6 +331,18 @@ impl SiblingsTraverser {
             .expect("[consistency] the node being traversed must be alive");
         neighbors.prev_sibling(hier)
     }
+
+    /// Returns the next event without advancing the iterator.
+    #[must_use]
+    pub(crate) fn peek(&self) -> Option<NodeId> {
+        self.next.map(|(forward, _back)| forward)
+    }
+
+    /// Returns the backward next event without advancing the iterator.
+    #[must_use]
+    pub(crate) fn peek_back(&self) -> Option<NodeId> {
+        self.next.map(|(_fwd, back)| back)
+    }
 }
 
 /// Limited depth-first tree traverser in "safe mode".
@@ -565,6 +596,18 @@ impl ShallowDepthFirstTraverser {
             }
         }
     }
+
+    /// Returns the next event without advancing the iterator.
+    #[must_use]
+    pub(crate) fn peek(&self) -> Option<(DftEvent, usize)> {
+        self.next.map(|(forward, _back)| forward)
+    }
+
+    /// Returns the backward next event without advancing the iterator.
+    #[must_use]
+    pub(crate) fn peek_back(&self) -> Option<(DftEvent, usize)> {
+        self.next.map(|(_fwd, back)| back)
+    }
 }
 
 /// Breadth-first tree traverser.
@@ -744,4 +787,36 @@ impl AllocatingBreadthFirstTraverser {
             (len - 1, None)
         }
     }
+
+    /// Returns the next event without advancing the iterator.
+    #[inline]
+    #[must_use]
+    pub(crate) fn peek(&self) -> Option<(NodeId, usize)> {
+        match *self.events.front()? {
+            BftQueuedEvent::Node(next) => Some((next, self.current_depth)),
+            BftQueuedEvent::IncrementDepth => match *self.events.get(1)? {
+                BftQueuedEvent::Node(next) => Some((next, self.current_depth + 1)),
+                BftQueuedEvent::IncrementDepth => panic!(
+                    "[consistency] `IncrementDepth` must not appear right after `IncrementDepth`"
+                ),
+            },
+        }
+    }
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    use core::mem;
+
+    #[test]
+    fn bft_queued_event_niche_optimized() {
+        assert_eq!(
+            mem::size_of::<BftQueuedEvent>(),
+            mem::size_of::<NodeId>(),
+            "`BftQueuedEvent` type must have the same size as \
+             `NodeId` type due to niche optimization"
+        );
+    }
 }
